import com.google.common.collect.LinkedListMultimap

// check dependency
if (!binding.variables.containsKey('zephyr')) {
	throw new Exception("The script \"zephyr-report-tool.groovy\" requires the prior execution of \"zephyr-tool.groovy\".\nUse this expression in one of the topmost cells: <mt:execute script=\"zephyr-tool.groovy\"/><mt:execute script=\"zephyr-report-tool.groovy\"/>")
}

zephyrReport = new zephyrReportTool(zephyr: zephyr)

class zephyrReportTool {
	def zephyr

	def getAssignee(issue) {
		issue.assignee?.displayName ?: '(Unassigned)'
	}

	def getExecutionVersion(testExecution) {
		testExecution.versionName ?: '(No version)'
	}

	def getTestCycle(testExecution) {
		testExecution.execution.cycleName
	}

	def getExecutionStatus(testExecution) {
		testExecution.execution.status.name
	}

	def getExecutionDefects(testExecution) {
		testExecution.execution.defectsAsString ?: '(No execution defects)'
	}

	def getStepDefects(testExecution) {
		testExecution.execution.stepDefectsAsString ?: '(No step defects)'
	}

	def getExecutedBy(testExecution) {
		testExecution.execution.executedBy ?: '(Unexecuted)'
	}

	def getExecutedOn(testExecution) {
		testExecution.execution.executedOn ?: '(Unexecuted)'
	}

	/**
	 * Returns a "repeating" list of entries where the issue (test) is the repeating key and its execution is the value.
	 * Example: (i1 -> e11, i1 -> e12, i1 -> e13, i2 -> e21, i3 -> e31, ...)
	 */
	def getRepeatingExecutions(issues) {
		def result = LinkedListMultimap.create() // keeps insertion order for both keys and values

		issues.each {
			it -> result.putAll(it, zephyr.getTestExecutions(it))
		}

		return result.entries()
	}
}